from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.workspace_create_request import WorkspaceCreateRequest
from ...models.workspace_response import WorkspaceResponse
from typing import cast



def _get_kwargs(
    *,
    body: WorkspaceCreateRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/workspace/create_protected",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | WorkspaceResponse | None:
    if response.status_code == 200:
        response_200 = WorkspaceResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | WorkspaceResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceCreateRequest,

) -> Response[HTTPValidationError | WorkspaceResponse]:
    """ Create Protected Workspace

     Create a new workspace with encryption and access controls.
    Sets up vector storage and associates the creator as the initial workspace user.

    Client generates the workspace symmetric key and sends it encrypted with the
    session key (SecretBox). Server decrypts and wraps it with the user's
    public key for storage. The wrapped key is returned in the response.

    Public workspaces are visible to all users and grant non-members limited access:
    - Non-members can view shared documents and tags
    - Non-members can create conversations and send messages
    - Only members can upload documents
    - Only members can see the member list

    Only users with developer flag can create public workspaces.

    Args:
        body (WorkspaceCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | WorkspaceResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceCreateRequest,

) -> HTTPValidationError | WorkspaceResponse | None:
    """ Create Protected Workspace

     Create a new workspace with encryption and access controls.
    Sets up vector storage and associates the creator as the initial workspace user.

    Client generates the workspace symmetric key and sends it encrypted with the
    session key (SecretBox). Server decrypts and wraps it with the user's
    public key for storage. The wrapped key is returned in the response.

    Public workspaces are visible to all users and grant non-members limited access:
    - Non-members can view shared documents and tags
    - Non-members can create conversations and send messages
    - Only members can upload documents
    - Only members can see the member list

    Only users with developer flag can create public workspaces.

    Args:
        body (WorkspaceCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | WorkspaceResponse
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceCreateRequest,

) -> Response[HTTPValidationError | WorkspaceResponse]:
    """ Create Protected Workspace

     Create a new workspace with encryption and access controls.
    Sets up vector storage and associates the creator as the initial workspace user.

    Client generates the workspace symmetric key and sends it encrypted with the
    session key (SecretBox). Server decrypts and wraps it with the user's
    public key for storage. The wrapped key is returned in the response.

    Public workspaces are visible to all users and grant non-members limited access:
    - Non-members can view shared documents and tags
    - Non-members can create conversations and send messages
    - Only members can upload documents
    - Only members can see the member list

    Only users with developer flag can create public workspaces.

    Args:
        body (WorkspaceCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | WorkspaceResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceCreateRequest,

) -> HTTPValidationError | WorkspaceResponse | None:
    """ Create Protected Workspace

     Create a new workspace with encryption and access controls.
    Sets up vector storage and associates the creator as the initial workspace user.

    Client generates the workspace symmetric key and sends it encrypted with the
    session key (SecretBox). Server decrypts and wraps it with the user's
    public key for storage. The wrapped key is returned in the response.

    Public workspaces are visible to all users and grant non-members limited access:
    - Non-members can view shared documents and tags
    - Non-members can create conversations and send messages
    - Only members can upload documents
    - Only members can see the member list

    Only users with developer flag can create public workspaces.

    Args:
        body (WorkspaceCreateRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | WorkspaceResponse
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from enum import Enum

class RetrievalChunkToolArgsSearchModeType0(str, Enum):
    HYBRID = "hybrid"
    KEYWORD = "keyword"
    SEMANTIC = "semantic"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast






T = TypeVar("T", bound="TableView")



@_attrs_define
class TableView:
    """ Saved column configuration for the document table.

    Column ID formats:
    - Standard columns: "doc_date", "title", "file_name", "status", "n_pages", "created_at"
    - Tags: tag external ID (e.g., "tag-a1b2c3d4")

        Attributes:
            workspace (str):
            name (str):
            columns (list[str]):
     """

    workspace: str
    name: str
    columns: list[str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        workspace = self.workspace

        name = self.name

        columns = self.columns




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "workspace": workspace,
            "name": name,
            "columns": columns,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        workspace = d.pop("workspace")

        name = d.pop("name")

        columns = cast(list[str], d.pop("columns"))


        table_view = cls(
            workspace=workspace,
            name=name,
            columns=columns,
        )


        table_view.additional_properties = d
        return table_view

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.chunk import Chunk
  from ..models.parsed_doc_footnotes import ParsedDocFootnotes
  from ..models.parsed_doc_metadata import ParsedDocMetadata





T = TypeVar("T", bound="ParsedDoc")



@_attrs_define
class ParsedDoc:
    """ 
        Attributes:
            metadata (ParsedDocMetadata):
            footnotes (ParsedDocFootnotes | Unset):
            chunks (list[Chunk] | Unset):
     """

    metadata: ParsedDocMetadata
    footnotes: ParsedDocFootnotes | Unset = UNSET
    chunks: list[Chunk] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.parsed_doc_metadata import ParsedDocMetadata
        from ..models.parsed_doc_footnotes import ParsedDocFootnotes
        from ..models.chunk import Chunk
        metadata = self.metadata.to_dict()

        footnotes: dict[str, Any] | Unset = UNSET
        if not isinstance(self.footnotes, Unset):
            footnotes = self.footnotes.to_dict()

        chunks: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.chunks, Unset):
            chunks = []
            for chunks_item_data in self.chunks:
                chunks_item = chunks_item_data.to_dict()
                chunks.append(chunks_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "metadata": metadata,
        })
        if footnotes is not UNSET:
            field_dict["footnotes"] = footnotes
        if chunks is not UNSET:
            field_dict["chunks"] = chunks

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.chunk import Chunk
        from ..models.parsed_doc_footnotes import ParsedDocFootnotes
        from ..models.parsed_doc_metadata import ParsedDocMetadata
        d = dict(src_dict)
        metadata = ParsedDocMetadata.from_dict(d.pop("metadata"))




        _footnotes = d.pop("footnotes", UNSET)
        footnotes: ParsedDocFootnotes | Unset
        if isinstance(_footnotes,  Unset):
            footnotes = UNSET
        else:
            footnotes = ParsedDocFootnotes.from_dict(_footnotes)




        _chunks = d.pop("chunks", UNSET)
        chunks: list[Chunk] | Unset = UNSET
        if _chunks is not UNSET:
            chunks = []
            for chunks_item_data in _chunks:
                chunks_item = Chunk.from_dict(chunks_item_data)



                chunks.append(chunks_item)


        parsed_doc = cls(
            metadata=metadata,
            footnotes=footnotes,
            chunks=chunks,
        )


        parsed_doc.additional_properties = d
        return parsed_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import Literal, cast






T = TypeVar("T", bound="CreatePlanDetail")



@_attrs_define
class CreatePlanDetail:
    """ Detail for a create_plan tool call.

        Attributes:
            query (str): Planning query (truncated to 80 chars)
            tool (Literal['create_plan'] | Unset):  Default: 'create_plan'.
     """

    query: str
    tool: Literal['create_plan'] | Unset = 'create_plan'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        query = self.query

        tool = self.tool


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "query": query,
        })
        if tool is not UNSET:
            field_dict["tool"] = tool

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        query = d.pop("query")

        tool = cast(Literal['create_plan'] | Unset , d.pop("tool", UNSET))
        if tool != 'create_plan'and not isinstance(tool, Unset):
            raise ValueError(f"tool must match const 'create_plan', got '{tool}'")

        create_plan_detail = cls(
            query=query,
            tool=tool,
        )


        create_plan_detail.additional_properties = d
        return create_plan_detail

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.register_request import RegisterRequest
from typing import cast



def _get_kwargs(
    *,
    body: RegisterRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/register",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Any | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[Any | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: RegisterRequest,

) -> Response[Any | HTTPValidationError]:
    """ Register User

     Register a new user with email verification (local or SSO).

    Accepts either:
    - 3-word verification code for local users
    - SSO JWT token for SSO users

    Auto-detects credential type and handles both flows.

    Args:
        body (RegisterRequest): Unified user registration request (local and SSO).

            Used by: /user/register

            Accepts either:
            - 3-word verification code (local users): "purple-mountain-river"
            - SSO JWT token (SSO users): "eyJ..." (long token with dots)

            For local users: given_name and family_name are required
            For SSO users: given_name and family_name are optional (extracted from JWT if not
            provided)

            Key derivation:
            1. Client derives Ed25519 keypair from password using Argon2id
            2. Client sends Ed25519 public key (signing_key)
            3. Backend derives X25519 encryption key from Ed25519 signing key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: RegisterRequest,

) -> Any | HTTPValidationError | None:
    """ Register User

     Register a new user with email verification (local or SSO).

    Accepts either:
    - 3-word verification code for local users
    - SSO JWT token for SSO users

    Auto-detects credential type and handles both flows.

    Args:
        body (RegisterRequest): Unified user registration request (local and SSO).

            Used by: /user/register

            Accepts either:
            - 3-word verification code (local users): "purple-mountain-river"
            - SSO JWT token (SSO users): "eyJ..." (long token with dots)

            For local users: given_name and family_name are required
            For SSO users: given_name and family_name are optional (extracted from JWT if not
            provided)

            Key derivation:
            1. Client derives Ed25519 keypair from password using Argon2id
            2. Client sends Ed25519 public key (signing_key)
            3. Backend derives X25519 encryption key from Ed25519 signing key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: RegisterRequest,

) -> Response[Any | HTTPValidationError]:
    """ Register User

     Register a new user with email verification (local or SSO).

    Accepts either:
    - 3-word verification code for local users
    - SSO JWT token for SSO users

    Auto-detects credential type and handles both flows.

    Args:
        body (RegisterRequest): Unified user registration request (local and SSO).

            Used by: /user/register

            Accepts either:
            - 3-word verification code (local users): "purple-mountain-river"
            - SSO JWT token (SSO users): "eyJ..." (long token with dots)

            For local users: given_name and family_name are required
            For SSO users: given_name and family_name are optional (extracted from JWT if not
            provided)

            Key derivation:
            1. Client derives Ed25519 keypair from password using Argon2id
            2. Client sends Ed25519 public key (signing_key)
            3. Backend derives X25519 encryption key from Ed25519 signing key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: RegisterRequest,

) -> Any | HTTPValidationError | None:
    """ Register User

     Register a new user with email verification (local or SSO).

    Accepts either:
    - 3-word verification code for local users
    - SSO JWT token for SSO users

    Auto-detects credential type and handles both flows.

    Args:
        body (RegisterRequest): Unified user registration request (local and SSO).

            Used by: /user/register

            Accepts either:
            - 3-word verification code (local users): "purple-mountain-river"
            - SSO JWT token (SSO users): "eyJ..." (long token with dots)

            For local users: given_name and family_name are required
            For SSO users: given_name and family_name are optional (extracted from JWT if not
            provided)

            Key derivation:
            1. Client derives Ed25519 keypair from password using Argon2id
            2. Client sends Ed25519 public key (signing_key)
            3. Backend derives X25519 encryption key from Ed25519 signing key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.product_price import ProductPrice





T = TypeVar("T", bound="ProductResponse")



@_attrs_define
class ProductResponse:
    """ Stripe product with pricing information.

        Attributes:
            product_id (str):
            name (str):
            prices (list[ProductPrice]):
            description (None | str | Unset):
     """

    product_id: str
    name: str
    prices: list[ProductPrice]
    description: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.product_price import ProductPrice
        product_id = self.product_id

        name = self.name

        prices = []
        for prices_item_data in self.prices:
            prices_item = prices_item_data.to_dict()
            prices.append(prices_item)



        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "product_id": product_id,
            "name": name,
            "prices": prices,
        })
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.product_price import ProductPrice
        d = dict(src_dict)
        product_id = d.pop("product_id")

        name = d.pop("name")

        prices = []
        _prices = d.pop("prices")
        for prices_item_data in (_prices):
            prices_item = ProductPrice.from_dict(prices_item_data)



            prices.append(prices_item)


        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))


        product_response = cls(
            product_id=product_id,
            name=name,
            prices=prices,
            description=description,
        )


        product_response.additional_properties = d
        return product_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Data types for the ARBI SDK."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any, TypeAlias

if TYPE_CHECKING:
    from arbi_client.models.agent_step_event import AgentStepEvent
    from arbi_client.models.artifact_event import ArtifactEvent
    from arbi_client.models.user_input_request_event import UserInputRequestEvent
    from arbi_client.models.user_message_event import UserMessageEvent

#: Union of all typed SSE event models (ARBI extension events only;
#: OpenAI standard events are handled as raw dicts).
SseEvent: TypeAlias = "AgentStepEvent | UserInputRequestEvent | ArtifactEvent | UserMessageEvent"


@dataclass
class UserIdentity:
    """Client-side user identity containing keys derived from password."""

    email: str
    external_id: str
    given_name: str
    family_name: str
    signing_private_key: bytes  # Ed25519 private key (64 bytes)
    signing_public_key_b64: str  # Ed25519 public key (base64)
    encryption_private_key: bytes  # X25519 private key (32 bytes)
    encryption_public_key_b64: str  # X25519 public key (base64)


@dataclass
class WorkspaceInfo:
    """Workspace metadata returned at login (for auto-caching headers)."""

    external_id: str
    wrapped_key: str | None = None  # X25519-wrapped workspace key (base64)


@dataclass
class SessionState:
    """Server session state after login."""

    access_token: str
    session_public_key_b64: str  # X25519 session public key (base64)
    wrapped_keys: dict[str, str] = field(default_factory=dict)  # workspace_ext_id -> X25519-wrapped key from DB (lazy)
    workspace_headers: dict[str, str] = field(
        default_factory=dict
    )  # workspace_ext_id -> SealedBox-encrypted header (computed on demand)


@dataclass
class StreamingEvent:
    """A single SSE event with its typed payload and timing.

    Attributes:
        event_type: The SSE event name (``"response.created"``, ``"arbi.agent_step"``, etc.).
        data: Parsed typed model for known events, raw dict/str for unknown.
        elapsed: Seconds since the HTTP stream started (client-side measurement).
    """

    event_type: str
    data: SseEvent | dict[str, Any] | str
    elapsed: float | None = None


@dataclass
class QueryResult:
    """Result of a streaming agent query.

    Attributes:
        content: Concatenated answer text from token events.
        events: All SSE events as :class:`StreamingEvent` instances.
        user_input_events: ``user_input_request`` events (typed).
        metadata: Terminal ``message_metadata`` payload, if received.
        assistant_message_ext_id: The message ID assigned by the server.
    """

    content: str
    events: list[StreamingEvent] = field(default_factory=list)
    user_input_events: list[UserInputRequestEvent] = field(default_factory=list)
    metadata: dict[str, Any] | None = None
    assistant_message_ext_id: str | None = None
    error: str | None = None



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="NotificationUpdate")



@_attrs_define
class NotificationUpdate:
    """ Single notification update for bulk PATCH.

    Supports two operations:
    - content: Re-encrypt content (key rotation)
    - read: Mark as read (only recipient can do this)

        Attributes:
            external_id (str):
            content (None | str | Unset):
            read (bool | None | Unset):
     """

    external_id: str
    content: None | str | Unset = UNSET
    read: bool | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        external_id = self.external_id

        content: None | str | Unset
        if isinstance(self.content, Unset):
            content = UNSET
        else:
            content = self.content

        read: bool | None | Unset
        if isinstance(self.read, Unset):
            read = UNSET
        else:
            read = self.read


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "external_id": external_id,
        })
        if content is not UNSET:
            field_dict["content"] = content
        if read is not UNSET:
            field_dict["read"] = read

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        external_id = d.pop("external_id")

        def _parse_content(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        content = _parse_content(d.pop("content", UNSET))


        def _parse_read(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        read = _parse_read(d.pop("read", UNSET))


        notification_update = cls(
            external_id=external_id,
            content=content,
            read=read,
        )


        notification_update.additional_properties = d
        return notification_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.artifact_event_mode import ArtifactEventMode
from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="ArtifactEvent")



@_attrs_define
class ArtifactEvent:
    r""" A generated artifact (document/draft) for the artifact panel.

    Artifacts ARE documents (wp_type='artifact') - the doc_ext_id is the single
    identifier used throughout. They support versioning for redline/track-changes
    functionality (useful for legal document review).

    The frontend should:
    - Open the artifact panel when receiving an artifact event
    - Stream content updates while is_complete=False
    - Enable redline view when multiple versions exist

    Examples:
        {"type": "arbi.artifact", "doc_ext_id": "doc-abc12345", "title": "Contract Summary",
         "mode": "markdown", "content": "# Summary\n...", "is_complete": false}
        {"type": "arbi.artifact", "doc_ext_id": "doc-abc12345", "title": "Contract Summary",
         "mode": "markdown", "content": "# Summary\n...", "is_complete": true, "version": 1}

        Attributes:
            doc_ext_id (str): Document external ID (artifacts are documents)
            title (str): Display title for the artifact
            content (str): The artifact content (markdown, html, or plain text)
            t (float | None | Unset): Seconds elapsed since stream start.
            type_ (Literal['arbi.artifact'] | Unset):  Default: 'arbi.artifact'.
            mode (ArtifactEventMode | Unset): Rendering mode: markdown (default, supports redlines), html, or plain text
                Default: ArtifactEventMode.MARKDOWN.
            is_complete (bool | Unset): Whether the artifact is fully generated (False during streaming) Default: False.
            version (int | Unset): Version number for tracking changes/redlines Default: 1.
            message_ext_id (None | str | Unset): External ID of the assistant message that generated this artifact
     """

    doc_ext_id: str
    title: str
    content: str
    t: float | None | Unset = UNSET
    type_: Literal['arbi.artifact'] | Unset = 'arbi.artifact'
    mode: ArtifactEventMode | Unset = ArtifactEventMode.MARKDOWN
    is_complete: bool | Unset = False
    version: int | Unset = 1
    message_ext_id: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_id = self.doc_ext_id

        title = self.title

        content = self.content

        t: float | None | Unset
        if isinstance(self.t, Unset):
            t = UNSET
        else:
            t = self.t

        type_ = self.type_

        mode: str | Unset = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value


        is_complete = self.is_complete

        version = self.version

        message_ext_id: None | str | Unset
        if isinstance(self.message_ext_id, Unset):
            message_ext_id = UNSET
        else:
            message_ext_id = self.message_ext_id


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "doc_ext_id": doc_ext_id,
            "title": title,
            "content": content,
        })
        if t is not UNSET:
            field_dict["t"] = t
        if type_ is not UNSET:
            field_dict["type"] = type_
        if mode is not UNSET:
            field_dict["mode"] = mode
        if is_complete is not UNSET:
            field_dict["is_complete"] = is_complete
        if version is not UNSET:
            field_dict["version"] = version
        if message_ext_id is not UNSET:
            field_dict["message_ext_id"] = message_ext_id

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_id = d.pop("doc_ext_id")

        title = d.pop("title")

        content = d.pop("content")

        def _parse_t(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        t = _parse_t(d.pop("t", UNSET))


        type_ = cast(Literal['arbi.artifact'] | Unset , d.pop("type", UNSET))
        if type_ != 'arbi.artifact'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'arbi.artifact', got '{type_}'")

        _mode = d.pop("mode", UNSET)
        mode: ArtifactEventMode | Unset
        if isinstance(_mode,  Unset):
            mode = UNSET
        else:
            mode = ArtifactEventMode(_mode)




        is_complete = d.pop("is_complete", UNSET)

        version = d.pop("version", UNSET)

        def _parse_message_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        message_ext_id = _parse_message_ext_id(d.pop("message_ext_id", UNSET))


        artifact_event = cls(
            doc_ext_id=doc_ext_id,
            title=title,
            content=content,
            t=t,
            type_=type_,
            mode=mode,
            is_complete=is_complete,
            version=version,
            message_ext_id=message_ext_id,
        )


        artifact_event.additional_properties = d
        return artifact_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.change_password_request import ChangePasswordRequest
from ...models.change_password_response import ChangePasswordResponse
from ...models.http_validation_error import HTTPValidationError
from typing import cast



def _get_kwargs(
    *,
    body: ChangePasswordRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/change_password",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> ChangePasswordResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = ChangePasswordResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[ChangePasswordResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: ChangePasswordRequest,

) -> Response[ChangePasswordResponse | HTTPValidationError]:
    r""" Change Password

     Change user's master password by re-keying all workspace keys.

    Self re-key (agent_ext_id omitted):
    1. Sign \"email|timestamp\" with current Ed25519 key (proves current password)
    2. Provide new Ed25519 signing key (derived from new password)
    3. Re-wrap all workspace keys with new X25519 public key

    Agent re-key (agent_ext_id provided):
    1. Parent signs \"agent_ext_id|timestamp\" with their own Ed25519 key (proves authority)
    2. Provide new Ed25519 signing key for the agent
    3. Re-wrap the agent's workspace keys with the agent's new X25519 public key

    Args:
        body (ChangePasswordRequest): Password change request with signature-based auth.

            Self re-key (agent_ext_id omitted):
              Client proves knowledge of current password by signing "email|timestamp"
              with their own Ed25519 key.

            Agent re-key (agent_ext_id provided):
              Parent re-keys an agent's signing/encryption keys.  Parent proves authority
              by signing "agent_ext_id|timestamp" with their own Ed25519 key.
              rewrapped_workspace_keys must contain entries for the agent's workspaces,
              each wrapped with the agent's new X25519 public key.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ChangePasswordResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: ChangePasswordRequest,

) -> ChangePasswordResponse | HTTPValidationError | None:
    r""" Change Password

     Change user's master password by re-keying all workspace keys.

    Self re-key (agent_ext_id omitted):
    1. Sign \"email|timestamp\" with current Ed25519 key (proves current password)
    2. Provide new Ed25519 signing key (derived from new password)
    3. Re-wrap all workspace keys with new X25519 public key

    Agent re-key (agent_ext_id provided):
    1. Parent signs \"agent_ext_id|timestamp\" with their own Ed25519 key (proves authority)
    2. Provide new Ed25519 signing key for the agent
    3. Re-wrap the agent's workspace keys with the agent's new X25519 public key

    Args:
        body (ChangePasswordRequest): Password change request with signature-based auth.

            Self re-key (agent_ext_id omitted):
              Client proves knowledge of current password by signing "email|timestamp"
              with their own Ed25519 key.

            Agent re-key (agent_ext_id provided):
              Parent re-keys an agent's signing/encryption keys.  Parent proves authority
              by signing "agent_ext_id|timestamp" with their own Ed25519 key.
              rewrapped_workspace_keys must contain entries for the agent's workspaces,
              each wrapped with the agent's new X25519 public key.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ChangePasswordResponse | HTTPValidationError
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: ChangePasswordRequest,

) -> Response[ChangePasswordResponse | HTTPValidationError]:
    r""" Change Password

     Change user's master password by re-keying all workspace keys.

    Self re-key (agent_ext_id omitted):
    1. Sign \"email|timestamp\" with current Ed25519 key (proves current password)
    2. Provide new Ed25519 signing key (derived from new password)
    3. Re-wrap all workspace keys with new X25519 public key

    Agent re-key (agent_ext_id provided):
    1. Parent signs \"agent_ext_id|timestamp\" with their own Ed25519 key (proves authority)
    2. Provide new Ed25519 signing key for the agent
    3. Re-wrap the agent's workspace keys with the agent's new X25519 public key

    Args:
        body (ChangePasswordRequest): Password change request with signature-based auth.

            Self re-key (agent_ext_id omitted):
              Client proves knowledge of current password by signing "email|timestamp"
              with their own Ed25519 key.

            Agent re-key (agent_ext_id provided):
              Parent re-keys an agent's signing/encryption keys.  Parent proves authority
              by signing "agent_ext_id|timestamp" with their own Ed25519 key.
              rewrapped_workspace_keys must contain entries for the agent's workspaces,
              each wrapped with the agent's new X25519 public key.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ChangePasswordResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: ChangePasswordRequest,

) -> ChangePasswordResponse | HTTPValidationError | None:
    r""" Change Password

     Change user's master password by re-keying all workspace keys.

    Self re-key (agent_ext_id omitted):
    1. Sign \"email|timestamp\" with current Ed25519 key (proves current password)
    2. Provide new Ed25519 signing key (derived from new password)
    3. Re-wrap all workspace keys with new X25519 public key

    Agent re-key (agent_ext_id provided):
    1. Parent signs \"agent_ext_id|timestamp\" with their own Ed25519 key (proves authority)
    2. Provide new Ed25519 signing key for the agent
    3. Re-wrap the agent's workspace keys with the agent's new X25519 public key

    Args:
        body (ChangePasswordRequest): Password change request with signature-based auth.

            Self re-key (agent_ext_id omitted):
              Client proves knowledge of current password by signing "email|timestamp"
              with their own Ed25519 key.

            Agent re-key (agent_ext_id provided):
              Parent re-keys an agent's signing/encryption keys.  Parent proves authority
              by signing "agent_ext_id|timestamp" with their own Ed25519 key.
              rewrapped_workspace_keys must contain entries for the agent's workspaces,
              each wrapped with the agent's new X25519 public key.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ChangePasswordResponse | HTTPValidationError
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="WorkspaceCreateRequest")



@_attrs_define
class WorkspaceCreateRequest:
    """ 
        Attributes:
            name (str):
            workspace_key (str):
            description (None | str | Unset):
            is_public (bool | Unset):  Default: False.
     """

    name: str
    workspace_key: str
    description: None | str | Unset = UNSET
    is_public: bool | Unset = False





    def to_dict(self) -> dict[str, Any]:
        name = self.name

        workspace_key = self.workspace_key

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        is_public = self.is_public


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "name": name,
            "workspace_key": workspace_key,
        })
        if description is not UNSET:
            field_dict["description"] = description
        if is_public is not UNSET:
            field_dict["is_public"] = is_public

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        workspace_key = d.pop("workspace_key")

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))


        is_public = d.pop("is_public", UNSET)

        workspace_create_request = cls(
            name=name,
            workspace_key=workspace_key,
            description=description,
            is_public=is_public,
        )

        return workspace_create_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="SubscriptionInfo")



@_attrs_define
class SubscriptionInfo:
    """ Subscription info exposed to frontend in user settings.

    Note: stripe_customer_id is deliberately excluded for security.
    This is a minimal model used in UserSettingsResponse.
    For full subscription details, use GET /subscription endpoint.

        Attributes:
            status (str | Unset):  Default: 'restricted'.
            trial_expires (int | None | Unset):
     """

    status: str | Unset = 'restricted'
    trial_expires: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        status = self.status

        trial_expires: int | None | Unset
        if isinstance(self.trial_expires, Unset):
            trial_expires = UNSET
        else:
            trial_expires = self.trial_expires


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if status is not UNSET:
            field_dict["status"] = status
        if trial_expires is not UNSET:
            field_dict["trial_expires"] = trial_expires

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status", UNSET)

        def _parse_trial_expires(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        trial_expires = _parse_trial_expires(d.pop("trial_expires", UNSET))


        subscription_info = cls(
            status=status,
            trial_expires=trial_expires,
        )


        subscription_info.additional_properties = d
        return subscription_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.models_health_status import ModelsHealthStatus
  from ..models.service_status import ServiceStatus





T = TypeVar("T", bound="ConsolidatedHealthResponse")



@_attrs_define
class ConsolidatedHealthResponse:
    """ Consolidated health response containing all system status and version information

        Attributes:
            status (str):
            backend_git_hash (None | str | Unset):
            frontend_docker_version (None | str | Unset):
            services (list[ServiceStatus] | Unset):
            models_health (ModelsHealthStatus | None | Unset):
            available_models (list[str] | Unset):
     """

    status: str
    backend_git_hash: None | str | Unset = UNSET
    frontend_docker_version: None | str | Unset = UNSET
    services: list[ServiceStatus] | Unset = UNSET
    models_health: ModelsHealthStatus | None | Unset = UNSET
    available_models: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.models_health_status import ModelsHealthStatus
        from ..models.service_status import ServiceStatus
        status = self.status

        backend_git_hash: None | str | Unset
        if isinstance(self.backend_git_hash, Unset):
            backend_git_hash = UNSET
        else:
            backend_git_hash = self.backend_git_hash

        frontend_docker_version: None | str | Unset
        if isinstance(self.frontend_docker_version, Unset):
            frontend_docker_version = UNSET
        else:
            frontend_docker_version = self.frontend_docker_version

        services: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.services, Unset):
            services = []
            for services_item_data in self.services:
                services_item = services_item_data.to_dict()
                services.append(services_item)



        models_health: dict[str, Any] | None | Unset
        if isinstance(self.models_health, Unset):
            models_health = UNSET
        elif isinstance(self.models_health, ModelsHealthStatus):
            models_health = self.models_health.to_dict()
        else:
            models_health = self.models_health

        available_models: list[str] | Unset = UNSET
        if not isinstance(self.available_models, Unset):
            available_models = self.available_models




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "status": status,
        })
        if backend_git_hash is not UNSET:
            field_dict["backend_git_hash"] = backend_git_hash
        if frontend_docker_version is not UNSET:
            field_dict["frontend_docker_version"] = frontend_docker_version
        if services is not UNSET:
            field_dict["services"] = services
        if models_health is not UNSET:
            field_dict["models_health"] = models_health
        if available_models is not UNSET:
            field_dict["available_models"] = available_models

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.models_health_status import ModelsHealthStatus
        from ..models.service_status import ServiceStatus
        d = dict(src_dict)
        status = d.pop("status")

        def _parse_backend_git_hash(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        backend_git_hash = _parse_backend_git_hash(d.pop("backend_git_hash", UNSET))


        def _parse_frontend_docker_version(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        frontend_docker_version = _parse_frontend_docker_version(d.pop("frontend_docker_version", UNSET))


        _services = d.pop("services", UNSET)
        services: list[ServiceStatus] | Unset = UNSET
        if _services is not UNSET:
            services = []
            for services_item_data in _services:
                services_item = ServiceStatus.from_dict(services_item_data)



                services.append(services_item)


        def _parse_models_health(data: object) -> ModelsHealthStatus | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                models_health_type_0 = ModelsHealthStatus.from_dict(data)



                return models_health_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ModelsHealthStatus | None | Unset, data)

        models_health = _parse_models_health(d.pop("models_health", UNSET))


        available_models = cast(list[str], d.pop("available_models", UNSET))


        consolidated_health_response = cls(
            status=status,
            backend_git_hash=backend_git_hash,
            frontend_docker_version=frontend_docker_version,
            services=services,
            models_health=models_health,
            available_models=available_models,
        )


        consolidated_health_response.additional_properties = d
        return consolidated_health_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.tag_format_type import TagFormatType
from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="TagFormat")



@_attrs_define
class TagFormat:
    """ Tag format configuration stored as JSONB.

    Type-specific fields:
    - select: options (list of choices, can be single or multi-select)
    - search: tag name is the query, chunks include relevance scores
    - checkbox, text, number: type only

        Attributes:
            type_ (TagFormatType | Unset):  Default: TagFormatType.CHECKBOX.
            options (list[str] | Unset):
     """

    type_: TagFormatType | Unset = TagFormatType.CHECKBOX
    options: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value


        options: list[str] | Unset = UNSET
        if not isinstance(self.options, Unset):
            options = self.options




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if type_ is not UNSET:
            field_dict["type"] = type_
        if options is not UNSET:
            field_dict["options"] = options

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _type_ = d.pop("type", UNSET)
        type_: TagFormatType | Unset
        if isinstance(_type_,  Unset):
            type_ = UNSET
        else:
            type_ = TagFormatType(_type_)




        options = cast(list[str], d.pop("options", UNSET))


        tag_format = cls(
            type_=type_,
            options=options,
        )


        tag_format.additional_properties = d
        return tag_format

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.remove_workspace_user_item import RemoveWorkspaceUserItem





T = TypeVar("T", bound="RemoveWorkspaceUsersRequest")



@_attrs_define
class RemoveWorkspaceUsersRequest:
    """ Request to remove users from a workspace (DELETE /workspace/{id}/users).

        Attributes:
            users (list[RemoveWorkspaceUserItem]):
     """

    users: list[RemoveWorkspaceUserItem]





    def to_dict(self) -> dict[str, Any]:
        from ..models.remove_workspace_user_item import RemoveWorkspaceUserItem
        users = []
        for users_item_data in self.users:
            users_item = users_item_data.to_dict()
            users.append(users_item)




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "users": users,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.remove_workspace_user_item import RemoveWorkspaceUserItem
        d = dict(src_dict)
        users = []
        _users = d.pop("users")
        for users_item_data in (_users):
            users_item = RemoveWorkspaceUserItem.from_dict(users_item_data)



            users.append(users_item)


        remove_workspace_users_request = cls(
            users=users,
        )

        return remove_workspace_users_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="FileObject")



@_attrs_define
class FileObject:
    """ OpenAI-compatible file object response.

    Matches the OpenAI FileObject schema:
    https://platform.openai.com/docs/api-reference/files/object

        Attributes:
            id (str):
            bytes_ (int):
            created_at (int):
            filename (str):
            object_ (str | Unset):  Default: 'file'.
            purpose (str | Unset):  Default: 'assistants'.
            status (str | Unset):  Default: 'uploaded'.
            expires_at (int | None | Unset):
            status_details (None | str | Unset):
     """

    id: str
    bytes_: int
    created_at: int
    filename: str
    object_: str | Unset = 'file'
    purpose: str | Unset = 'assistants'
    status: str | Unset = 'uploaded'
    expires_at: int | None | Unset = UNSET
    status_details: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        id = self.id

        bytes_ = self.bytes_

        created_at = self.created_at

        filename = self.filename

        object_ = self.object_

        purpose = self.purpose

        status = self.status

        expires_at: int | None | Unset
        if isinstance(self.expires_at, Unset):
            expires_at = UNSET
        else:
            expires_at = self.expires_at

        status_details: None | str | Unset
        if isinstance(self.status_details, Unset):
            status_details = UNSET
        else:
            status_details = self.status_details


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "bytes": bytes_,
            "created_at": created_at,
            "filename": filename,
        })
        if object_ is not UNSET:
            field_dict["object"] = object_
        if purpose is not UNSET:
            field_dict["purpose"] = purpose
        if status is not UNSET:
            field_dict["status"] = status
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at
        if status_details is not UNSET:
            field_dict["status_details"] = status_details

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        bytes_ = d.pop("bytes")

        created_at = d.pop("created_at")

        filename = d.pop("filename")

        object_ = d.pop("object", UNSET)

        purpose = d.pop("purpose", UNSET)

        status = d.pop("status", UNSET)

        def _parse_expires_at(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        expires_at = _parse_expires_at(d.pop("expires_at", UNSET))


        def _parse_status_details(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        status_details = _parse_status_details(d.pop("status_details", UNSET))


        file_object = cls(
            id=id,
            bytes_=bytes_,
            created_at=created_at,
            filename=filename,
            object_=object_,
            purpose=purpose,
            status=status,
            expires_at=expires_at,
            status_details=status_details,
        )


        file_object.additional_properties = d
        return file_object

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.login_response_workspaces_item import LoginResponseWorkspacesItem
  from ..models.user_response import UserResponse





T = TypeVar("T", bound="LoginResponse")



@_attrs_define
class LoginResponse:
    """ Login response with access token, session key, user info, and workspaces.

    Used by: /user/login, /user/token (claim agent session)

        Attributes:
            access_token (str):
            session_key (str):
            user (UserResponse): Standard user representation used across all endpoints.

                Used for: login response, workspace users, contacts (when registered).
                Agents have agt- prefix and may have nullable name/email fields.
            workspaces (list[LoginResponseWorkspacesItem] | Unset):
     """

    access_token: str
    session_key: str
    user: UserResponse
    workspaces: list[LoginResponseWorkspacesItem] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.user_response import UserResponse
        from ..models.login_response_workspaces_item import LoginResponseWorkspacesItem
        access_token = self.access_token

        session_key = self.session_key

        user = self.user.to_dict()

        workspaces: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.workspaces, Unset):
            workspaces = []
            for workspaces_item_data in self.workspaces:
                workspaces_item = workspaces_item_data.to_dict()
                workspaces.append(workspaces_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "access_token": access_token,
            "session_key": session_key,
            "user": user,
        })
        if workspaces is not UNSET:
            field_dict["workspaces"] = workspaces

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.login_response_workspaces_item import LoginResponseWorkspacesItem
        from ..models.user_response import UserResponse
        d = dict(src_dict)
        access_token = d.pop("access_token")

        session_key = d.pop("session_key")

        user = UserResponse.from_dict(d.pop("user"))




        _workspaces = d.pop("workspaces", UNSET)
        workspaces: list[LoginResponseWorkspacesItem] | Unset = UNSET
        if _workspaces is not UNSET:
            workspaces = []
            for workspaces_item_data in _workspaces:
                workspaces_item = LoginResponseWorkspacesItem.from_dict(workspaces_item_data)



                workspaces.append(workspaces_item)


        login_response = cls(
            access_token=access_token,
            session_key=session_key,
            user=user,
            workspaces=workspaces,
        )


        login_response.additional_properties = d
        return login_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.reranker_config_api_type import RerankerConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="RerankerConfig")



@_attrs_define
class RerankerConfig:
    """ 
        Attributes:
            min_score (float | Unset): Minimum reranker score for a chunk to be kept. With the Qwen3 strict fact-matching
                instruction, relevant chunks score 0.1+ and noise is typically < 0.005. Default: 0.01.
            max_numb_of_chunks (int | Unset): Maximum number of chunks to return after reranking. Default: 30.
            max_concurrent_requests (int | Unset): Maximum number of concurrent reranking requests. Default: 64.
            model_name (str | Unset): Name of the reranking model to use. Default: 'Qwen/Qwen3-Reranker-0.6B'.
            api_type (RerankerConfigApiType | Unset): The inference type (local or remote). Default:
                RerankerConfigApiType.LOCAL.
            retrieval_instruction (str | Unset): Instruction for the <Instruct> tag when reranking retrieval results against
                a user query. Default: 'Judge whether the passage contains the specific facts, figures, names, or references
                mentioned in the query. Only answer yes if the passage directly contains these details'.
     """

    min_score: float | Unset = 0.01
    max_numb_of_chunks: int | Unset = 30
    max_concurrent_requests: int | Unset = 64
    model_name: str | Unset = 'Qwen/Qwen3-Reranker-0.6B'
    api_type: RerankerConfigApiType | Unset = RerankerConfigApiType.LOCAL
    retrieval_instruction: str | Unset = 'Judge whether the passage contains the specific facts, figures, names, or references mentioned in the query. Only answer yes if the passage directly contains these details'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        min_score = self.min_score

        max_numb_of_chunks = self.max_numb_of_chunks

        max_concurrent_requests = self.max_concurrent_requests

        model_name = self.model_name

        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        retrieval_instruction = self.retrieval_instruction


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if min_score is not UNSET:
            field_dict["MIN_SCORE"] = min_score
        if max_numb_of_chunks is not UNSET:
            field_dict["MAX_NUMB_OF_CHUNKS"] = max_numb_of_chunks
        if max_concurrent_requests is not UNSET:
            field_dict["MAX_CONCURRENT_REQUESTS"] = max_concurrent_requests
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if retrieval_instruction is not UNSET:
            field_dict["RETRIEVAL_INSTRUCTION"] = retrieval_instruction

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        min_score = d.pop("MIN_SCORE", UNSET)

        max_numb_of_chunks = d.pop("MAX_NUMB_OF_CHUNKS", UNSET)

        max_concurrent_requests = d.pop("MAX_CONCURRENT_REQUESTS", UNSET)

        model_name = d.pop("MODEL_NAME", UNSET)

        _api_type = d.pop("API_TYPE", UNSET)
        api_type: RerankerConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = RerankerConfigApiType(_api_type)




        retrieval_instruction = d.pop("RETRIEVAL_INSTRUCTION", UNSET)

        reranker_config = cls(
            min_score=min_score,
            max_numb_of_chunks=max_numb_of_chunks,
            max_concurrent_requests=max_concurrent_requests,
            model_name=model_name,
            api_type=api_type,
            retrieval_instruction=retrieval_instruction,
        )


        reranker_config.additional_properties = d
        return reranker_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.upload_documents_response import UploadDocumentsResponse
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    *,
    urls: list[str],
    shared: bool | Unset = True,
    config_ext_id: None | str | Unset = UNSET,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    json_urls = urls


    params["urls"] = json_urls

    params["shared"] = shared

    json_config_ext_id: None | str | Unset
    if isinstance(config_ext_id, Unset):
        json_config_ext_id = UNSET
    else:
        json_config_ext_id = config_ext_id
    params["config_ext_id"] = json_config_ext_id


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/document/upload-url",
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | UploadDocumentsResponse | None:
    if response.status_code == 200:
        response_200 = UploadDocumentsResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | UploadDocumentsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    urls: list[str],
    shared: bool | Unset = True,
    config_ext_id: None | str | Unset = UNSET,

) -> Response[HTTPValidationError | UploadDocumentsResponse]:
    """ Upload Documents From Url

     Download and upload documents from URLs to a workspace with encryption.
    Documents are queued for processing, parsed, and indexed for vector search.

    Requires active subscription (paid/trial/dev) if Stripe is configured.

    Args:
        urls (list[str]): URLs to download documents from
        shared (bool | Unset): Whether the document should be shared with workspace members
            Default: True.
        config_ext_id (None | str | Unset): Configuration to use for processing

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | UploadDocumentsResponse]
     """


    kwargs = _get_kwargs(
        urls=urls,
shared=shared,
config_ext_id=config_ext_id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    urls: list[str],
    shared: bool | Unset = True,
    config_ext_id: None | str | Unset = UNSET,

) -> HTTPValidationError | UploadDocumentsResponse | None:
    """ Upload Documents From Url

     Download and upload documents from URLs to a workspace with encryption.
    Documents are queued for processing, parsed, and indexed for vector search.

    Requires active subscription (paid/trial/dev) if Stripe is configured.

    Args:
        urls (list[str]): URLs to download documents from
        shared (bool | Unset): Whether the document should be shared with workspace members
            Default: True.
        config_ext_id (None | str | Unset): Configuration to use for processing

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | UploadDocumentsResponse
     """


    return sync_detailed(
        client=client,
urls=urls,
shared=shared,
config_ext_id=config_ext_id,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    urls: list[str],
    shared: bool | Unset = True,
    config_ext_id: None | str | Unset = UNSET,

) -> Response[HTTPValidationError | UploadDocumentsResponse]:
    """ Upload Documents From Url

     Download and upload documents from URLs to a workspace with encryption.
    Documents are queued for processing, parsed, and indexed for vector search.

    Requires active subscription (paid/trial/dev) if Stripe is configured.

    Args:
        urls (list[str]): URLs to download documents from
        shared (bool | Unset): Whether the document should be shared with workspace members
            Default: True.
        config_ext_id (None | str | Unset): Configuration to use for processing

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | UploadDocumentsResponse]
     """


    kwargs = _get_kwargs(
        urls=urls,
shared=shared,
config_ext_id=config_ext_id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    urls: list[str],
    shared: bool | Unset = True,
    config_ext_id: None | str | Unset = UNSET,

) -> HTTPValidationError | UploadDocumentsResponse | None:
    """ Upload Documents From Url

     Download and upload documents from URLs to a workspace with encryption.
    Documents are queued for processing, parsed, and indexed for vector search.

    Requires active subscription (paid/trial/dev) if Stripe is configured.

    Args:
        urls (list[str]): URLs to download documents from
        shared (bool | Unset): Whether the document should be shared with workspace members
            Default: True.
        config_ext_id (None | str | Unset): Configuration to use for processing

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | UploadDocumentsResponse
     """


    return (await asyncio_detailed(
        client=client,
urls=urls,
shared=shared,
config_ext_id=config_ext_id,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="SSOConfigResponse")



@_attrs_define
class SSOConfigResponse:
    """ Public SSO configuration for device flow / CLI login.

        Attributes:
            sso_enabled (bool):
            domain (str | Unset):  Default: ''.
            client_id (str | Unset):  Default: ''.
            cli_client_id (str | Unset):  Default: ''.
            audience (str | Unset):  Default: ''.
     """

    sso_enabled: bool
    domain: str | Unset = ''
    client_id: str | Unset = ''
    cli_client_id: str | Unset = ''
    audience: str | Unset = ''
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        sso_enabled = self.sso_enabled

        domain = self.domain

        client_id = self.client_id

        cli_client_id = self.cli_client_id

        audience = self.audience


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "sso_enabled": sso_enabled,
        })
        if domain is not UNSET:
            field_dict["domain"] = domain
        if client_id is not UNSET:
            field_dict["client_id"] = client_id
        if cli_client_id is not UNSET:
            field_dict["cli_client_id"] = cli_client_id
        if audience is not UNSET:
            field_dict["audience"] = audience

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        sso_enabled = d.pop("sso_enabled")

        domain = d.pop("domain", UNSET)

        client_id = d.pop("client_id", UNSET)

        cli_client_id = d.pop("cli_client_id", UNSET)

        audience = d.pop("audience", UNSET)

        sso_config_response = cls(
            sso_enabled=sso_enabled,
            domain=domain,
            client_id=client_id,
            cli_client_id=cli_client_id,
            audience=audience,
        )


        sso_config_response.additional_properties = d
        return sso_config_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast

if TYPE_CHECKING:
  from ..models.output_text_content import OutputTextContent





T = TypeVar("T", bound="ResponseContentPartAddedEvent")



@_attrs_define
class ResponseContentPartAddedEvent:
    """ OpenAI response.content_part.added: announces a new content part.

        Attributes:
            sequence_number (int): Monotonic sequence number for event ordering
            item_id (str): ID of the parent output message
            part (OutputTextContent): A text content block in an output message.
            type_ (Literal['response.content_part.added'] | Unset):  Default: 'response.content_part.added'.
            output_index (int | Unset):  Default: 0.
            content_index (int | Unset):  Default: 0.
     """

    sequence_number: int
    item_id: str
    part: OutputTextContent
    type_: Literal['response.content_part.added'] | Unset = 'response.content_part.added'
    output_index: int | Unset = 0
    content_index: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.output_text_content import OutputTextContent
        sequence_number = self.sequence_number

        item_id = self.item_id

        part = self.part.to_dict()

        type_ = self.type_

        output_index = self.output_index

        content_index = self.content_index


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "sequence_number": sequence_number,
            "item_id": item_id,
            "part": part,
        })
        if type_ is not UNSET:
            field_dict["type"] = type_
        if output_index is not UNSET:
            field_dict["output_index"] = output_index
        if content_index is not UNSET:
            field_dict["content_index"] = content_index

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.output_text_content import OutputTextContent
        d = dict(src_dict)
        sequence_number = d.pop("sequence_number")

        item_id = d.pop("item_id")

        part = OutputTextContent.from_dict(d.pop("part"))




        type_ = cast(Literal['response.content_part.added'] | Unset , d.pop("type", UNSET))
        if type_ != 'response.content_part.added'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'response.content_part.added', got '{type_}'")

        output_index = d.pop("output_index", UNSET)

        content_index = d.pop("content_index", UNSET)

        response_content_part_added_event = cls(
            sequence_number=sequence_number,
            item_id=item_id,
            part=part,
            type_=type_,
            output_index=output_index,
            content_index=content_index,
        )


        response_content_part_added_event.additional_properties = d
        return response_content_part_added_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.create_subscription_request import CreateSubscriptionRequest
from ...models.create_subscription_response import CreateSubscriptionResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    *,
    body: CreateSubscriptionRequest,
    origin: str | Unset = UNSET,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(origin, Unset):
        headers["origin"] = origin



    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/subscription",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> CreateSubscriptionResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = CreateSubscriptionResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[CreateSubscriptionResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: CreateSubscriptionRequest,
    origin: str | Unset = UNSET,

) -> Response[CreateSubscriptionResponse | HTTPValidationError]:
    """ Create Subscription

     Create a new Stripe checkout session for subscription purchase.

    Args:
        origin (str | Unset):
        body (CreateSubscriptionRequest): Request to create a new subscription checkout session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CreateSubscriptionResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,
origin=origin,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: CreateSubscriptionRequest,
    origin: str | Unset = UNSET,

) -> CreateSubscriptionResponse | HTTPValidationError | None:
    """ Create Subscription

     Create a new Stripe checkout session for subscription purchase.

    Args:
        origin (str | Unset):
        body (CreateSubscriptionRequest): Request to create a new subscription checkout session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CreateSubscriptionResponse | HTTPValidationError
     """


    return sync_detailed(
        client=client,
body=body,
origin=origin,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: CreateSubscriptionRequest,
    origin: str | Unset = UNSET,

) -> Response[CreateSubscriptionResponse | HTTPValidationError]:
    """ Create Subscription

     Create a new Stripe checkout session for subscription purchase.

    Args:
        origin (str | Unset):
        body (CreateSubscriptionRequest): Request to create a new subscription checkout session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[CreateSubscriptionResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,
origin=origin,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: CreateSubscriptionRequest,
    origin: str | Unset = UNSET,

) -> CreateSubscriptionResponse | HTTPValidationError | None:
    """ Create Subscription

     Create a new Stripe checkout session for subscription purchase.

    Args:
        origin (str | Unset):
        body (CreateSubscriptionRequest): Request to create a new subscription checkout session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        CreateSubscriptionResponse | HTTPValidationError
     """


    return (await asyncio_detailed(
        client=client,
body=body,
origin=origin,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="CreateSubscriptionResponse")



@_attrs_define
class CreateSubscriptionResponse:
    """ Response containing Stripe checkout session client secret.

        Attributes:
            client_secret (str):
     """

    client_secret: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        client_secret = self.client_secret


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "client_secret": client_secret,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        client_secret = d.pop("client_secret")

        create_subscription_response = cls(
            client_secret=client_secret,
        )


        create_subscription_response.additional_properties = d
        return create_subscription_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="KeywordEmbedderConfig")



@_attrs_define
class KeywordEmbedderConfig:
    """ Configuration for keyword embedder with BM25 scoring.

        Attributes:
            dimension_space (int | Unset): Total dimension space for hash trick (1,048,576 dimensions) Default: 1048576.
            filter_stopwords (bool | Unset): Remove common stopwords to reduce noise Default: True.
            bm25_k1 (float | Unset): BM25 term frequency saturation (1.2-2.0). Higher = more weight on term repetition.
                Default 1.5 works for most cases. Default: 1.5.
            bm25_b (float | Unset): BM25 document length normalization (0.0-1.0). 0=ignore length, 1=full penalty for long
                docs. Default 0.75 is standard. Default: 0.75.
            bm25_avgdl (float | Unset): Average document length in tokens. Adjust based on your documents: chat messages
                ~20-50, articles ~100-300, papers ~1000+ Default: 100.0.
     """

    dimension_space: int | Unset = 1048576
    filter_stopwords: bool | Unset = True
    bm25_k1: float | Unset = 1.5
    bm25_b: float | Unset = 0.75
    bm25_avgdl: float | Unset = 100.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        dimension_space = self.dimension_space

        filter_stopwords = self.filter_stopwords

        bm25_k1 = self.bm25_k1

        bm25_b = self.bm25_b

        bm25_avgdl = self.bm25_avgdl


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if dimension_space is not UNSET:
            field_dict["DIMENSION_SPACE"] = dimension_space
        if filter_stopwords is not UNSET:
            field_dict["FILTER_STOPWORDS"] = filter_stopwords
        if bm25_k1 is not UNSET:
            field_dict["BM25_K1"] = bm25_k1
        if bm25_b is not UNSET:
            field_dict["BM25_B"] = bm25_b
        if bm25_avgdl is not UNSET:
            field_dict["BM25_AVGDL"] = bm25_avgdl

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        dimension_space = d.pop("DIMENSION_SPACE", UNSET)

        filter_stopwords = d.pop("FILTER_STOPWORDS", UNSET)

        bm25_k1 = d.pop("BM25_K1", UNSET)

        bm25_b = d.pop("BM25_B", UNSET)

        bm25_avgdl = d.pop("BM25_AVGDL", UNSET)

        keyword_embedder_config = cls(
            dimension_space=dimension_space,
            filter_stopwords=filter_stopwords,
            bm25_k1=bm25_k1,
            bm25_b=bm25_b,
            bm25_avgdl=bm25_avgdl,
        )


        keyword_embedder_config.additional_properties = d
        return keyword_embedder_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime






T = TypeVar("T", bound="ConversationResponse")



@_attrs_define
class ConversationResponse:
    """ 
        Attributes:
            external_id (str):
            title (None | str):
            created_by_ext_id (str):
            created_at (datetime.datetime):
            updated_at (datetime.datetime):
            message_count (int):
            updated_by_ext_id (None | str | Unset):
            is_shared (bool | Unset):  Default: False.
            last_message_status (None | str | Unset):
     """

    external_id: str
    title: None | str
    created_by_ext_id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    message_count: int
    updated_by_ext_id: None | str | Unset = UNSET
    is_shared: bool | Unset = False
    last_message_status: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        external_id = self.external_id

        title: None | str
        title = self.title

        created_by_ext_id = self.created_by_ext_id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        message_count = self.message_count

        updated_by_ext_id: None | str | Unset
        if isinstance(self.updated_by_ext_id, Unset):
            updated_by_ext_id = UNSET
        else:
            updated_by_ext_id = self.updated_by_ext_id

        is_shared = self.is_shared

        last_message_status: None | str | Unset
        if isinstance(self.last_message_status, Unset):
            last_message_status = UNSET
        else:
            last_message_status = self.last_message_status


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "external_id": external_id,
            "title": title,
            "created_by_ext_id": created_by_ext_id,
            "created_at": created_at,
            "updated_at": updated_at,
            "message_count": message_count,
        })
        if updated_by_ext_id is not UNSET:
            field_dict["updated_by_ext_id"] = updated_by_ext_id
        if is_shared is not UNSET:
            field_dict["is_shared"] = is_shared
        if last_message_status is not UNSET:
            field_dict["last_message_status"] = last_message_status

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        external_id = d.pop("external_id")

        def _parse_title(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        title = _parse_title(d.pop("title"))


        created_by_ext_id = d.pop("created_by_ext_id")

        created_at = isoparse(d.pop("created_at"))




        updated_at = isoparse(d.pop("updated_at"))




        message_count = d.pop("message_count")

        def _parse_updated_by_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        updated_by_ext_id = _parse_updated_by_ext_id(d.pop("updated_by_ext_id", UNSET))


        is_shared = d.pop("is_shared", UNSET)

        def _parse_last_message_status(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        last_message_status = _parse_last_message_status(d.pop("last_message_status", UNSET))


        conversation_response = cls(
            external_id=external_id,
            title=title,
            created_by_ext_id=created_by_ext_id,
            created_at=created_at,
            updated_at=updated_at,
            message_count=message_count,
            updated_by_ext_id=updated_by_ext_id,
            is_shared=is_shared,
            last_message_status=last_message_status,
        )


        conversation_response.additional_properties = d
        return conversation_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.change_password_request_rewrapped_agent_recovery_keys_type_0 import ChangePasswordRequestRewrappedAgentRecoveryKeysType0
  from ..models.change_password_request_rewrapped_workspace_keys import ChangePasswordRequestRewrappedWorkspaceKeys





T = TypeVar("T", bound="ChangePasswordRequest")



@_attrs_define
class ChangePasswordRequest:
    """ Password change request with signature-based auth.

    Self re-key (agent_ext_id omitted):
      Client proves knowledge of current password by signing "email|timestamp"
      with their own Ed25519 key.

    Agent re-key (agent_ext_id provided):
      Parent re-keys an agent's signing/encryption keys.  Parent proves authority
      by signing "agent_ext_id|timestamp" with their own Ed25519 key.
      rewrapped_workspace_keys must contain entries for the agent's workspaces,
      each wrapped with the agent's new X25519 public key.

        Attributes:
            signature (str):
            timestamp (int):
            new_signing_key (str):
            rewrapped_workspace_keys (ChangePasswordRequestRewrappedWorkspaceKeys):
            agent_ext_id (None | str | Unset):
            rewrapped_agent_recovery_keys (ChangePasswordRequestRewrappedAgentRecoveryKeysType0 | None | Unset):
     """

    signature: str
    timestamp: int
    new_signing_key: str
    rewrapped_workspace_keys: ChangePasswordRequestRewrappedWorkspaceKeys
    agent_ext_id: None | str | Unset = UNSET
    rewrapped_agent_recovery_keys: ChangePasswordRequestRewrappedAgentRecoveryKeysType0 | None | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.change_password_request_rewrapped_workspace_keys import ChangePasswordRequestRewrappedWorkspaceKeys
        from ..models.change_password_request_rewrapped_agent_recovery_keys_type_0 import ChangePasswordRequestRewrappedAgentRecoveryKeysType0
        signature = self.signature

        timestamp = self.timestamp

        new_signing_key = self.new_signing_key

        rewrapped_workspace_keys = self.rewrapped_workspace_keys.to_dict()

        agent_ext_id: None | str | Unset
        if isinstance(self.agent_ext_id, Unset):
            agent_ext_id = UNSET
        else:
            agent_ext_id = self.agent_ext_id

        rewrapped_agent_recovery_keys: dict[str, Any] | None | Unset
        if isinstance(self.rewrapped_agent_recovery_keys, Unset):
            rewrapped_agent_recovery_keys = UNSET
        elif isinstance(self.rewrapped_agent_recovery_keys, ChangePasswordRequestRewrappedAgentRecoveryKeysType0):
            rewrapped_agent_recovery_keys = self.rewrapped_agent_recovery_keys.to_dict()
        else:
            rewrapped_agent_recovery_keys = self.rewrapped_agent_recovery_keys


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "signature": signature,
            "timestamp": timestamp,
            "new_signing_key": new_signing_key,
            "rewrapped_workspace_keys": rewrapped_workspace_keys,
        })
        if agent_ext_id is not UNSET:
            field_dict["agent_ext_id"] = agent_ext_id
        if rewrapped_agent_recovery_keys is not UNSET:
            field_dict["rewrapped_agent_recovery_keys"] = rewrapped_agent_recovery_keys

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.change_password_request_rewrapped_agent_recovery_keys_type_0 import ChangePasswordRequestRewrappedAgentRecoveryKeysType0
        from ..models.change_password_request_rewrapped_workspace_keys import ChangePasswordRequestRewrappedWorkspaceKeys
        d = dict(src_dict)
        signature = d.pop("signature")

        timestamp = d.pop("timestamp")

        new_signing_key = d.pop("new_signing_key")

        rewrapped_workspace_keys = ChangePasswordRequestRewrappedWorkspaceKeys.from_dict(d.pop("rewrapped_workspace_keys"))




        def _parse_agent_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        agent_ext_id = _parse_agent_ext_id(d.pop("agent_ext_id", UNSET))


        def _parse_rewrapped_agent_recovery_keys(data: object) -> ChangePasswordRequestRewrappedAgentRecoveryKeysType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                rewrapped_agent_recovery_keys_type_0 = ChangePasswordRequestRewrappedAgentRecoveryKeysType0.from_dict(data)



                return rewrapped_agent_recovery_keys_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChangePasswordRequestRewrappedAgentRecoveryKeysType0 | None | Unset, data)

        rewrapped_agent_recovery_keys = _parse_rewrapped_agent_recovery_keys(d.pop("rewrapped_agent_recovery_keys", UNSET))


        change_password_request = cls(
            signature=signature,
            timestamp=timestamp,
            new_signing_key=new_signing_key,
            rewrapped_workspace_keys=rewrapped_workspace_keys,
            agent_ext_id=agent_ext_id,
            rewrapped_agent_recovery_keys=rewrapped_agent_recovery_keys,
        )

        return change_password_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="UserInputBody")



@_attrs_define
class UserInputBody:
    """ Request body for POST /assistant/respond.

        Attributes:
            assistant_message_ext_id (str):
            answer (str):
     """

    assistant_message_ext_id: str
    answer: str





    def to_dict(self) -> dict[str, Any]:
        assistant_message_ext_id = self.assistant_message_ext_id

        answer = self.answer


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "assistant_message_ext_id": assistant_message_ext_id,
            "answer": answer,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        assistant_message_ext_id = d.pop("assistant_message_ext_id")

        answer = d.pop("answer")

        user_input_body = cls(
            assistant_message_ext_id=assistant_message_ext_id,
            answer=answer,
        )

        return user_input_body




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.workspace_role import WorkspaceRole
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime

if TYPE_CHECKING:
  from ..models.user_response import UserResponse





T = TypeVar("T", bound="WorkspaceUserResponse")



@_attrs_define
class WorkspaceUserResponse:
    """ User (or agent) with their role in a workspace.

    For agents: agent_ext_id is set (agt-XXXXXXXX), name is user.given_name.
    is_temporary: True when the grant is session-scoped (expires_at is set in DB).

        Attributes:
            user (UserResponse): Standard user representation used across all endpoints.

                Used for: login response, workspace users, contacts (when registered).
                Agents have agt- prefix and may have nullable name/email fields.
            role (WorkspaceRole): Role of a user within a workspace.
            joined_at (datetime.datetime):
            conversation_count (int | Unset):  Default: 0.
            document_count (int | Unset):  Default: 0.
            agent_ext_id (None | str | Unset):
            is_temporary (bool | Unset):  Default: False.
     """

    user: UserResponse
    role: WorkspaceRole
    joined_at: datetime.datetime
    conversation_count: int | Unset = 0
    document_count: int | Unset = 0
    agent_ext_id: None | str | Unset = UNSET
    is_temporary: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.user_response import UserResponse
        user = self.user.to_dict()

        role = self.role.value

        joined_at = self.joined_at.isoformat()

        conversation_count = self.conversation_count

        document_count = self.document_count

        agent_ext_id: None | str | Unset
        if isinstance(self.agent_ext_id, Unset):
            agent_ext_id = UNSET
        else:
            agent_ext_id = self.agent_ext_id

        is_temporary = self.is_temporary


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "user": user,
            "role": role,
            "joined_at": joined_at,
        })
        if conversation_count is not UNSET:
            field_dict["conversation_count"] = conversation_count
        if document_count is not UNSET:
            field_dict["document_count"] = document_count
        if agent_ext_id is not UNSET:
            field_dict["agent_ext_id"] = agent_ext_id
        if is_temporary is not UNSET:
            field_dict["is_temporary"] = is_temporary

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user_response import UserResponse
        d = dict(src_dict)
        user = UserResponse.from_dict(d.pop("user"))




        role = WorkspaceRole(d.pop("role"))




        joined_at = isoparse(d.pop("joined_at"))




        conversation_count = d.pop("conversation_count", UNSET)

        document_count = d.pop("document_count", UNSET)

        def _parse_agent_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        agent_ext_id = _parse_agent_ext_id(d.pop("agent_ext_id", UNSET))


        is_temporary = d.pop("is_temporary", UNSET)

        workspace_user_response = cls(
            user=user,
            role=role,
            joined_at=joined_at,
            conversation_count=conversation_count,
            document_count=document_count,
            agent_ext_id=agent_ext_id,
            is_temporary=is_temporary,
        )


        workspace_user_response.additional_properties = d
        return workspace_user_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast






T = TypeVar("T", bound="DeleteDocumentsRequest")



@_attrs_define
class DeleteDocumentsRequest:
    """ Delete one or more documents by external ID.

        Attributes:
            external_ids (list[str]):
     """

    external_ids: list[str]





    def to_dict(self) -> dict[str, Any]:
        external_ids = self.external_ids




        field_dict: dict[str, Any] = {}

        field_dict.update({
            "external_ids": external_ids,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        external_ids = cast(list[str], d.pop("external_ids"))


        delete_documents_request = cls(
            external_ids=external_ids,
        )

        return delete_documents_request




from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.workspace_open_request import WorkspaceOpenRequest
from ...models.workspace_open_response import WorkspaceOpenResponse
from typing import cast



def _get_kwargs(
    workspace_ext_id: str,
    *,
    body: WorkspaceOpenRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/workspace/{workspace_ext_id}/open".format(workspace_ext_id=quote(str(workspace_ext_id), safe=""),),
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | WorkspaceOpenResponse | None:
    if response.status_code == 200:
        response_200 = WorkspaceOpenResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | WorkspaceOpenResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    workspace_ext_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceOpenRequest,

) -> Response[HTTPValidationError | WorkspaceOpenResponse]:
    """ Open Workspace

     Open a workspace: store encrypted workspace key in session, set active, return workspace data.

    Users provide ``workspace_key`` (SealedBox-encrypted with session public key).
    Agents omit it — their workspace keys are pre-populated at login from their WorkspaceUsers rows.

    Args:
        workspace_ext_id (str):
        body (WorkspaceOpenRequest): Request to open a workspace — stores encrypted key in session
            and sets active.

            workspace_key is required for users (SealedBox-encrypted with session public key).
            Agents omit it — their workspace keys are pre-populated via /agent-session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | WorkspaceOpenResponse]
     """


    kwargs = _get_kwargs(
        workspace_ext_id=workspace_ext_id,
body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    workspace_ext_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceOpenRequest,

) -> HTTPValidationError | WorkspaceOpenResponse | None:
    """ Open Workspace

     Open a workspace: store encrypted workspace key in session, set active, return workspace data.

    Users provide ``workspace_key`` (SealedBox-encrypted with session public key).
    Agents omit it — their workspace keys are pre-populated at login from their WorkspaceUsers rows.

    Args:
        workspace_ext_id (str):
        body (WorkspaceOpenRequest): Request to open a workspace — stores encrypted key in session
            and sets active.

            workspace_key is required for users (SealedBox-encrypted with session public key).
            Agents omit it — their workspace keys are pre-populated via /agent-session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | WorkspaceOpenResponse
     """


    return sync_detailed(
        workspace_ext_id=workspace_ext_id,
client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    workspace_ext_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceOpenRequest,

) -> Response[HTTPValidationError | WorkspaceOpenResponse]:
    """ Open Workspace

     Open a workspace: store encrypted workspace key in session, set active, return workspace data.

    Users provide ``workspace_key`` (SealedBox-encrypted with session public key).
    Agents omit it — their workspace keys are pre-populated at login from their WorkspaceUsers rows.

    Args:
        workspace_ext_id (str):
        body (WorkspaceOpenRequest): Request to open a workspace — stores encrypted key in session
            and sets active.

            workspace_key is required for users (SealedBox-encrypted with session public key).
            Agents omit it — their workspace keys are pre-populated via /agent-session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | WorkspaceOpenResponse]
     """


    kwargs = _get_kwargs(
        workspace_ext_id=workspace_ext_id,
body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    workspace_ext_id: str,
    *,
    client: AuthenticatedClient | Client,
    body: WorkspaceOpenRequest,

) -> HTTPValidationError | WorkspaceOpenResponse | None:
    """ Open Workspace

     Open a workspace: store encrypted workspace key in session, set active, return workspace data.

    Users provide ``workspace_key`` (SealedBox-encrypted with session public key).
    Agents omit it — their workspace keys are pre-populated at login from their WorkspaceUsers rows.

    Args:
        workspace_ext_id (str):
        body (WorkspaceOpenRequest): Request to open a workspace — stores encrypted key in session
            and sets active.

            workspace_key is required for users (SealedBox-encrypted with session public key).
            Agents omit it — their workspace keys are pre-populated via /agent-session.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | WorkspaceOpenResponse
     """


    return (await asyncio_detailed(
        workspace_ext_id=workspace_ext_id,
client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast






T = TypeVar("T", bound="GenerateAnnotationsResponse")



@_attrs_define
class GenerateAnnotationsResponse:
    """ Response for generate annotations endpoint (202 Accepted).

    Returns the accepted doc and tag IDs that will be processed.

        Attributes:
            doc_ext_ids (list[str]): Document IDs that will be processed
            tag_ext_ids (list[str]): Tag IDs that will be processed
     """

    doc_ext_ids: list[str]
    tag_ext_ids: list[str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_ids = self.doc_ext_ids



        tag_ext_ids = self.tag_ext_ids




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "doc_ext_ids": doc_ext_ids,
            "tag_ext_ids": tag_ext_ids,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids"))


        tag_ext_ids = cast(list[str], d.pop("tag_ext_ids"))


        generate_annotations_response = cls(
            doc_ext_ids=doc_ext_ids,
            tag_ext_ids=tag_ext_ids,
        )


        generate_annotations_response.additional_properties = d
        return generate_annotations_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.title_llm_config_api_type import TitleLLMConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="TitleLLMConfig")



@_attrs_define
class TitleLLMConfig:
    """ 
        Attributes:
            api_type (TitleLLMConfigApiType | Unset): The inference type (local or remote). Default:
                TitleLLMConfigApiType.REMOTE.
            enable_thinking (bool | Unset): Enable reasoning/thinking mode. Default: False.
            model_name (str | Unset): The name of the non-reasoning model to be used. Default: 'dummy'.
            system_instruction (str | Unset):  Default: 'You are a title generator that creates concise, descriptive
                titles.'.
            max_char_size_to_answer (int | Unset): Maximum character size to answer. Default: 50.
            temperature (float | Unset): Temperature value for randomness. Default: 0.3.
            max_tokens (int | Unset): Maximum number of tokens allowed. Default: 20.
     """

    api_type: TitleLLMConfigApiType | Unset = TitleLLMConfigApiType.REMOTE
    enable_thinking: bool | Unset = False
    model_name: str | Unset = 'dummy'
    system_instruction: str | Unset = 'You are a title generator that creates concise, descriptive titles.'
    max_char_size_to_answer: int | Unset = 50
    temperature: float | Unset = 0.3
    max_tokens: int | Unset = 20
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        enable_thinking = self.enable_thinking

        model_name = self.model_name

        system_instruction = self.system_instruction

        max_char_size_to_answer = self.max_char_size_to_answer

        temperature = self.temperature

        max_tokens = self.max_tokens


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if enable_thinking is not UNSET:
            field_dict["ENABLE_THINKING"] = enable_thinking
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if system_instruction is not UNSET:
            field_dict["SYSTEM_INSTRUCTION"] = system_instruction
        if max_char_size_to_answer is not UNSET:
            field_dict["MAX_CHAR_SIZE_TO_ANSWER"] = max_char_size_to_answer
        if temperature is not UNSET:
            field_dict["TEMPERATURE"] = temperature
        if max_tokens is not UNSET:
            field_dict["MAX_TOKENS"] = max_tokens

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _api_type = d.pop("API_TYPE", UNSET)
        api_type: TitleLLMConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = TitleLLMConfigApiType(_api_type)




        enable_thinking = d.pop("ENABLE_THINKING", UNSET)

        model_name = d.pop("MODEL_NAME", UNSET)

        system_instruction = d.pop("SYSTEM_INSTRUCTION", UNSET)

        max_char_size_to_answer = d.pop("MAX_CHAR_SIZE_TO_ANSWER", UNSET)

        temperature = d.pop("TEMPERATURE", UNSET)

        max_tokens = d.pop("MAX_TOKENS", UNSET)

        title_llm_config = cls(
            api_type=api_type,
            enable_thinking=enable_thinking,
            model_name=model_name,
            system_instruction=system_instruction,
            max_char_size_to_answer=max_char_size_to_answer,
            temperature=temperature,
            max_tokens=max_tokens,
        )


        title_llm_config.additional_properties = d
        return title_llm_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.agents_config_agent_api_type import AgentsConfigAgentApiType
from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.run_code_config import RunCodeConfig
  from ..models.web_search_config import WebSearchConfig





T = TypeVar("T", bound="AgentsConfig")



@_attrs_define
class AgentsConfig:
    r""" Configuration for agent workflow.

        Attributes:
            enabled (bool | Unset): Whether to use agents mode for queries. Default: True.
            human_in_the_loop (bool | Unset): Enable the ask_user tool, allowing the agent to pause and ask the user a
                question with options. Default: False.
            web_search (WebSearchConfig | Unset): Configuration for web search tools.
            run_code (RunCodeConfig | Unset): Configuration for the run_code agent tool (Docker sandbox).
            mcp_tools (list[str] | Unset): MCP tool names to enable from LiteLLM (e.g. ['read_wiki_contents',
                'ask_question']).
            planning_enabled (bool | Unset): Include the create_plan tool so the agent can generate a research plan when it
                deems a task complex enough. Plan approval is configured on PlanningLLM. Default: False.
            suggested_queries (bool | Unset): Generate a single suggested follow-up query after each response. Default:
                False.
            artifacts_enabled (bool | Unset): Enable the create_artifact tool for generating standalone documents
                (summaries, reports, drafts) displayed in a side panel. Default: False.
            vision_enabled (bool | Unset): Enable the view_document_pages tool for visual inspection of document pages
                (figures, charts, scanned pages) using a vision-capable model. Default: False.
            personal_agent (bool | Unset): Enable the personal_agent tool for delegating tasks to an external agent via
                OpenClaw. Default: False.
            memory_enabled (bool | Unset): Run memory synthesis (skills, memories) after agent queries. Default: False.
            skills_enabled (bool | Unset): Enable the skills system: /skill (create) and /<skill-name> (invoke) commands.
                Default: False.
            skill_creation (bool | Unset): Transient flag set by /skill command to enable the save_skill agent tool.
                Default: False.
            review_enabled (bool | Unset): Enable ReviewLLM to review agent draft answers against source material before
                sending to the user. Uses the reasoning model for higher quality. Default: False.
            persona (str | Unset): Agent persona prepended to the system prompt. Customise to change the agent's identity
                and tone. Default: 'You are ARBI, an AI assistant created by ARBI CITY. Be proactive, helpful and
                professional.'.
            agent_model_name (str | Unset): The name of the model to be used for the agent decision-making. Default: 'auto'.
            agent_api_type (AgentsConfigAgentApiType | Unset): The inference type for the agent LLM (local or remote).
                Default: AgentsConfigAgentApiType.REMOTE.
            llm_agent_temperature (float | Unset): Temperature value for agent LLM. Default: 1.0.
            agent_max_tokens (int | Unset): Maximum tokens for the agent LLM response. Default: 10000.
            enable_thinking (bool | Unset): Enable reasoning/thinking mode for the agentic loop LLM. Default: False.
            agent_max_iterations (int | Unset): Maximum number of tool call iterations before forcing an answer. Default:
                20.
            agent_max_parallel_tool_calls (int | Unset): Maximum number of tool calls the agent can make in a single
                iteration. Extra calls beyond this limit are silently dropped. Default: 10.
            agent_history_char_threshold (int | Unset): Char threshold for conversation history injected into the agent
                context. Below: verbatim user/assistant pairs. Above: summarized by SummariseLLM. Default: 8000.
            agent_system_prompt (str | Unset): System prompt for the agent. Dynamic context is appended automatically.
                Default: 'Answer any question the user asks. You have access to workspace documents — use them when relevant.
                When no documents are relevant, answer from your own knowledge. Always provide a helpful answer.\n\nIMPORTANT:
                Never say \\"I don\'t have information in the available documents\\" or refuse to answer. If documents are not
                relevant, just answer the question directly from your own knowledge.\n\nWhen using documents, retrieve actual
                document content — do not rely on metadata or headings alone. Cross-check information across sources. Not all
                assertions in sources are factual — some are arguments by a party. Attribute claims to their source where
                parties disagree.\n\nDOCUMENT INDEX:\nThe \\"Workspace Documents\\" table in your system prompt already lists
                every document with its Id, Title, Date, Pages, and Subject. Use this table directly when the user asks to list,
                catalogue, or tabulate documents — do NOT call any tools for that. Only use tools when you need to read or
                search document content.\n\nDOCUMENT STRATEGY:\n- For listing/cataloguing documents: use the document index
                already in your prompt — no tool calls needed.\n- For overview/summary requests: use get_table_of_contents
                first, then read key sections with read_document.\n- For specific questions: use search_inside_documents to find
                relevant sections, then read those sections with read_document.\n- For translation, deep analysis, or detailed
                summaries of a single document: use read_document with no ranges to load the complete text.\n- For extracting
                structured findings from passages: use read_document with digest=true or search_inside_documents with
                digest=true.\n- Never search for meta-queries like \\"summary\\", \\"overview\\", or \\"all documents\\" —
                search for the actual subject matter.\n\nFORMATTING:\n- Start your answer directly with the content — never open
                with a description of what you searched for or read (e.g. never write \\"Searching for X\\", \\"Reading Y\\",
                \\"I found Z documents\\" as the first lines of an answer).\n- Use markdown: headings, bold, bullet points,
                numbered lists\n- Main conclusion first, followed by supporting points\n- Bullet points for lists of discrete
                items\n- Numbering for sequential steps or prioritized items\n- Bold text for critical conclusions or decision
                points\n\nEFFICIENCY:\n1. Before each step, review the results of prior steps. Adapt your approach based on what
                you learned — if a call was rejected or returned poor results, adjust your strategy.\n2. Call multiple tools in
                parallel when a query has several aspects.\n3. NEVER repeat a tool call with the same arguments.\n4. Use
                targeted queries and narrow page ranges per retrieval call.\n5. Stop when you have sufficient evidence — do not
                over-research.'.
     """

    enabled: bool | Unset = True
    human_in_the_loop: bool | Unset = False
    web_search: WebSearchConfig | Unset = UNSET
    run_code: RunCodeConfig | Unset = UNSET
    mcp_tools: list[str] | Unset = UNSET
    planning_enabled: bool | Unset = False
    suggested_queries: bool | Unset = False
    artifacts_enabled: bool | Unset = False
    vision_enabled: bool | Unset = False
    personal_agent: bool | Unset = False
    memory_enabled: bool | Unset = False
    skills_enabled: bool | Unset = False
    skill_creation: bool | Unset = False
    review_enabled: bool | Unset = False
    persona: str | Unset = 'You are ARBI, an AI assistant created by ARBI CITY. Be proactive, helpful and professional.'
    agent_model_name: str | Unset = 'auto'
    agent_api_type: AgentsConfigAgentApiType | Unset = AgentsConfigAgentApiType.REMOTE
    llm_agent_temperature: float | Unset = 1.0
    agent_max_tokens: int | Unset = 10000
    enable_thinking: bool | Unset = False
    agent_max_iterations: int | Unset = 20
    agent_max_parallel_tool_calls: int | Unset = 10
    agent_history_char_threshold: int | Unset = 8000
    agent_system_prompt: str | Unset = 'Answer any question the user asks. You have access to workspace documents — use them when relevant. When no documents are relevant, answer from your own knowledge. Always provide a helpful answer.\n\nIMPORTANT: Never say \\"I don\'t have information in the available documents\\" or refuse to answer. If documents are not relevant, just answer the question directly from your own knowledge.\n\nWhen using documents, retrieve actual document content — do not rely on metadata or headings alone. Cross-check information across sources. Not all assertions in sources are factual — some are arguments by a party. Attribute claims to their source where parties disagree.\n\nDOCUMENT INDEX:\nThe \\"Workspace Documents\\" table in your system prompt already lists every document with its Id, Title, Date, Pages, and Subject. Use this table directly when the user asks to list, catalogue, or tabulate documents — do NOT call any tools for that. Only use tools when you need to read or search document content.\n\nDOCUMENT STRATEGY:\n- For listing/cataloguing documents: use the document index already in your prompt — no tool calls needed.\n- For overview/summary requests: use get_table_of_contents first, then read key sections with read_document.\n- For specific questions: use search_inside_documents to find relevant sections, then read those sections with read_document.\n- For translation, deep analysis, or detailed summaries of a single document: use read_document with no ranges to load the complete text.\n- For extracting structured findings from passages: use read_document with digest=true or search_inside_documents with digest=true.\n- Never search for meta-queries like \\"summary\\", \\"overview\\", or \\"all documents\\" — search for the actual subject matter.\n\nFORMATTING:\n- Start your answer directly with the content — never open with a description of what you searched for or read (e.g. never write \\"Searching for X\\", \\"Reading Y\\", \\"I found Z documents\\" as the first lines of an answer).\n- Use markdown: headings, bold, bullet points, numbered lists\n- Main conclusion first, followed by supporting points\n- Bullet points for lists of discrete items\n- Numbering for sequential steps or prioritized items\n- Bold text for critical conclusions or decision points\n\nEFFICIENCY:\n1. Before each step, review the results of prior steps. Adapt your approach based on what you learned — if a call was rejected or returned poor results, adjust your strategy.\n2. Call multiple tools in parallel when a query has several aspects.\n3. NEVER repeat a tool call with the same arguments.\n4. Use targeted queries and narrow page ranges per retrieval call.\n5. Stop when you have sufficient evidence — do not over-research.'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.web_search_config import WebSearchConfig
        from ..models.run_code_config import RunCodeConfig
        enabled = self.enabled

        human_in_the_loop = self.human_in_the_loop

        web_search: dict[str, Any] | Unset = UNSET
        if not isinstance(self.web_search, Unset):
            web_search = self.web_search.to_dict()

        run_code: dict[str, Any] | Unset = UNSET
        if not isinstance(self.run_code, Unset):
            run_code = self.run_code.to_dict()

        mcp_tools: list[str] | Unset = UNSET
        if not isinstance(self.mcp_tools, Unset):
            mcp_tools = self.mcp_tools



        planning_enabled = self.planning_enabled

        suggested_queries = self.suggested_queries

        artifacts_enabled = self.artifacts_enabled

        vision_enabled = self.vision_enabled

        personal_agent = self.personal_agent

        memory_enabled = self.memory_enabled

        skills_enabled = self.skills_enabled

        skill_creation = self.skill_creation

        review_enabled = self.review_enabled

        persona = self.persona

        agent_model_name = self.agent_model_name

        agent_api_type: str | Unset = UNSET
        if not isinstance(self.agent_api_type, Unset):
            agent_api_type = self.agent_api_type.value


        llm_agent_temperature = self.llm_agent_temperature

        agent_max_tokens = self.agent_max_tokens

        enable_thinking = self.enable_thinking

        agent_max_iterations = self.agent_max_iterations

        agent_max_parallel_tool_calls = self.agent_max_parallel_tool_calls

        agent_history_char_threshold = self.agent_history_char_threshold

        agent_system_prompt = self.agent_system_prompt


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if enabled is not UNSET:
            field_dict["ENABLED"] = enabled
        if human_in_the_loop is not UNSET:
            field_dict["HUMAN_IN_THE_LOOP"] = human_in_the_loop
        if web_search is not UNSET:
            field_dict["WEB_SEARCH"] = web_search
        if run_code is not UNSET:
            field_dict["RUN_CODE"] = run_code
        if mcp_tools is not UNSET:
            field_dict["MCP_TOOLS"] = mcp_tools
        if planning_enabled is not UNSET:
            field_dict["PLANNING_ENABLED"] = planning_enabled
        if suggested_queries is not UNSET:
            field_dict["SUGGESTED_QUERIES"] = suggested_queries
        if artifacts_enabled is not UNSET:
            field_dict["ARTIFACTS_ENABLED"] = artifacts_enabled
        if vision_enabled is not UNSET:
            field_dict["VISION_ENABLED"] = vision_enabled
        if personal_agent is not UNSET:
            field_dict["PERSONAL_AGENT"] = personal_agent
        if memory_enabled is not UNSET:
            field_dict["MEMORY_ENABLED"] = memory_enabled
        if skills_enabled is not UNSET:
            field_dict["SKILLS_ENABLED"] = skills_enabled
        if skill_creation is not UNSET:
            field_dict["SKILL_CREATION"] = skill_creation
        if review_enabled is not UNSET:
            field_dict["REVIEW_ENABLED"] = review_enabled
        if persona is not UNSET:
            field_dict["PERSONA"] = persona
        if agent_model_name is not UNSET:
            field_dict["AGENT_MODEL_NAME"] = agent_model_name
        if agent_api_type is not UNSET:
            field_dict["AGENT_API_TYPE"] = agent_api_type
        if llm_agent_temperature is not UNSET:
            field_dict["LLM_AGENT_TEMPERATURE"] = llm_agent_temperature
        if agent_max_tokens is not UNSET:
            field_dict["AGENT_MAX_TOKENS"] = agent_max_tokens
        if enable_thinking is not UNSET:
            field_dict["ENABLE_THINKING"] = enable_thinking
        if agent_max_iterations is not UNSET:
            field_dict["AGENT_MAX_ITERATIONS"] = agent_max_iterations
        if agent_max_parallel_tool_calls is not UNSET:
            field_dict["AGENT_MAX_PARALLEL_TOOL_CALLS"] = agent_max_parallel_tool_calls
        if agent_history_char_threshold is not UNSET:
            field_dict["AGENT_HISTORY_CHAR_THRESHOLD"] = agent_history_char_threshold
        if agent_system_prompt is not UNSET:
            field_dict["AGENT_SYSTEM_PROMPT"] = agent_system_prompt

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.run_code_config import RunCodeConfig
        from ..models.web_search_config import WebSearchConfig
        d = dict(src_dict)
        enabled = d.pop("ENABLED", UNSET)

        human_in_the_loop = d.pop("HUMAN_IN_THE_LOOP", UNSET)

        _web_search = d.pop("WEB_SEARCH", UNSET)
        web_search: WebSearchConfig | Unset
        if isinstance(_web_search,  Unset):
            web_search = UNSET
        else:
            web_search = WebSearchConfig.from_dict(_web_search)




        _run_code = d.pop("RUN_CODE", UNSET)
        run_code: RunCodeConfig | Unset
        if isinstance(_run_code,  Unset):
            run_code = UNSET
        else:
            run_code = RunCodeConfig.from_dict(_run_code)




        mcp_tools = cast(list[str], d.pop("MCP_TOOLS", UNSET))


        planning_enabled = d.pop("PLANNING_ENABLED", UNSET)

        suggested_queries = d.pop("SUGGESTED_QUERIES", UNSET)

        artifacts_enabled = d.pop("ARTIFACTS_ENABLED", UNSET)

        vision_enabled = d.pop("VISION_ENABLED", UNSET)

        personal_agent = d.pop("PERSONAL_AGENT", UNSET)

        memory_enabled = d.pop("MEMORY_ENABLED", UNSET)

        skills_enabled = d.pop("SKILLS_ENABLED", UNSET)

        skill_creation = d.pop("SKILL_CREATION", UNSET)

        review_enabled = d.pop("REVIEW_ENABLED", UNSET)

        persona = d.pop("PERSONA", UNSET)

        agent_model_name = d.pop("AGENT_MODEL_NAME", UNSET)

        _agent_api_type = d.pop("AGENT_API_TYPE", UNSET)
        agent_api_type: AgentsConfigAgentApiType | Unset
        if isinstance(_agent_api_type,  Unset):
            agent_api_type = UNSET
        else:
            agent_api_type = AgentsConfigAgentApiType(_agent_api_type)




        llm_agent_temperature = d.pop("LLM_AGENT_TEMPERATURE", UNSET)

        agent_max_tokens = d.pop("AGENT_MAX_TOKENS", UNSET)

        enable_thinking = d.pop("ENABLE_THINKING", UNSET)

        agent_max_iterations = d.pop("AGENT_MAX_ITERATIONS", UNSET)

        agent_max_parallel_tool_calls = d.pop("AGENT_MAX_PARALLEL_TOOL_CALLS", UNSET)

        agent_history_char_threshold = d.pop("AGENT_HISTORY_CHAR_THRESHOLD", UNSET)

        agent_system_prompt = d.pop("AGENT_SYSTEM_PROMPT", UNSET)

        agents_config = cls(
            enabled=enabled,
            human_in_the_loop=human_in_the_loop,
            web_search=web_search,
            run_code=run_code,
            mcp_tools=mcp_tools,
            planning_enabled=planning_enabled,
            suggested_queries=suggested_queries,
            artifacts_enabled=artifacts_enabled,
            vision_enabled=vision_enabled,
            personal_agent=personal_agent,
            memory_enabled=memory_enabled,
            skills_enabled=skills_enabled,
            skill_creation=skill_creation,
            review_enabled=review_enabled,
            persona=persona,
            agent_model_name=agent_model_name,
            agent_api_type=agent_api_type,
            llm_agent_temperature=llm_agent_temperature,
            agent_max_tokens=agent_max_tokens,
            enable_thinking=enable_thinking,
            agent_max_iterations=agent_max_iterations,
            agent_max_parallel_tool_calls=agent_max_parallel_tool_calls,
            agent_history_char_threshold=agent_history_char_threshold,
            agent_system_prompt=agent_system_prompt,
        )


        agents_config.additional_properties = d
        return agents_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""ARBI Python SDK — client-side crypto, auth, sessions, WebSocket, and streaming."""

from arbi_client.sdk import crypto, streaming, tools
from arbi_client.sdk._types import QueryResult, SessionState, SseEvent, StreamingEvent, UserIdentity
from arbi_client.sdk.session import ArbiSession, AsyncArbiSession
from arbi_client.sdk.streaming import SseEventPrinter, parse_sse_event
from arbi_client.sdk.websocket import (
    TERMINAL_STATUSES,
    ArbiWebSocket,
    WsMessage,
    parse_ws_message,
    print_ws_status,
)

__all__ = [
    "ArbiSession",
    "AsyncArbiSession",
    "ArbiWebSocket",
    "QueryResult",
    "SessionState",
    "SseEvent",
    "StreamingEvent",
    "TERMINAL_STATUSES",
    "UserIdentity",
    "WsMessage",
    "crypto",
    "SseEventPrinter",
    "parse_sse_event",
    "parse_ws_message",
    "print_ws_status",
    "streaming",
    "tools",
]
